package net.ihe.gazelle.proxy.netty.basictls;

import java.util.ArrayList;
import java.util.List;

import javax.net.ssl.KeyManager;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLEngine;
import javax.net.ssl.TrustManager;

import net.ihe.gazelle.proxy.netty.ChannelType;
import net.ihe.gazelle.proxy.netty.ConnectionConfigSimple;

import org.jboss.netty.channel.ChannelHandler;
import org.jboss.netty.handler.ssl.SslHandler;

public class ConnectionConfigSimpleTls<A, B> extends ConnectionConfigSimple<A, B> {

	private static final String PROTOCOL = "TLS";
	private static final TrustManager[] ALWAYS_TRUST_MANAGER = new TrustManager[] { new AlwaysTrustManager() };

	private SSLContext clientContext;
	private SSLContext serverContext;

	public ConnectionConfigSimpleTls(int localPort, String remoteHost, int remotePort, ChannelType channelType,
			TlsConfig tlsConfig) {
		super(localPort, remoteHost, remotePort, channelType);
		initContexts(tlsConfig);
	}

	@Override
	public List<ChannelHandler> getRequestConnectionHandlers() {
		List<ChannelHandler> result = new ArrayList<ChannelHandler>();

		if (serverContext != null) {
			SSLEngine engine = serverContext.createSSLEngine();
			engine.setUseClientMode(false);
			result.add(getChannelHandler(engine));
		}

		result.addAll(super.getRequestConnectionHandlers());
		return result;
	}

	@Override
	public List<ChannelHandler> getResponseConnectionHandlers() {
		List<ChannelHandler> result = new ArrayList<ChannelHandler>();

		if (clientContext != null) {
			SSLEngine engine = clientContext.createSSLEngine();
			engine.setUseClientMode(true);
			result.add(getChannelHandler(engine));
		}

		result.addAll(super.getResponseConnectionHandlers());
		return result;
	}

	public void initContexts(TlsConfig tlsConfig) {
		clientContext = createContext(tlsConfig.getClientCredentials());
		serverContext = createContext(tlsConfig.getServerCredentials());
	}

	private SSLContext createContext(TlsCredentials credentials) {
		SSLContext context = null;
		if (credentials != null) {
			try {
				context = SSLContext.getInstance(PROTOCOL);
				KeyManager keyManager = new TlsKeyManager(credentials);
				KeyManager[] keyManagers = new KeyManager[] { keyManager };
				context.init(keyManagers, ALWAYS_TRUST_MANAGER, null);
			} catch (Exception e) {
				throw new Error("Failed to initialize the SSLContext", e);
			}
		}
		return context;
	}

	private ChannelHandler getChannelHandler(SSLEngine engine) {
		SslHandler sslHandler = new SslHandler(engine);
		sslHandler.setEnableRenegotiation(true);
		return sslHandler;
	}

}
